<?php

require_once __DIR__ . '/../../const.php';

use Bitrix\Main\Page\Asset;
use Bitrix\Main\Web\Json;
use Bitrix\Sale\Payment;

IncludeModuleLangFile(__DIR__ . '/other.php');

CModule::IncludeModule(FERMAOFD_FERMA_MODULE_ID);

class CMainOfdFerma
{
    /**
     * @param \CAdminSaleList|\CAdminList $list
     */
    public static function onAdminListDisplay($list)
    {
        /** @var \CMain $APPLICATION */
        global $APPLICATION;

        if ($APPLICATION->GetCurPage() === '/bitrix/admin/sale_order.php') {
            Asset::getInstance()->addJs('/bitrix/js/' . FERMAOFD_FERMA_MODULE_ID . '/ofdferma_receipts.js');

            foreach ($list->aRows as $row) {
                $actions = $row->aActions;

                $newActions = [
                    [
                        'ICON' => 'edit',
                        'TEXT' => GetMessage('FERMAOFD_NEW_INCOME_RECEIPT'),
                        'ACTION' => 'window.ofdferma.openOrderReceiptPanel("' . $row->id . '");',
                    ], [
                        'ICON' => 'edit',
                        'TEXT' => GetMessage('FERMAOFD_NEW_OUTCOME_RECEIPT'),
                        'ACTION' => 'window.ofdferma.openOrderRefundPanel("' . $row->id . '");',
                    ],
                ];

                array_splice($actions, -2, 0, $newActions);

                $row->AddActions($actions);
            }
        }
    }

    public static function onAdminContextMenuShow(&$items)
    {
        /** @var \CMain $APPLICATION */
        global $APPLICATION;

        if ($APPLICATION->GetCurPage() === '/bitrix/admin/sale_order_view.php') {
            Asset::getInstance()->addJs('/bitrix/js/' . FERMAOFD_FERMA_MODULE_ID . '/ofdferma_receipts.js');

            $orderId = (int)$_GET['ID'];

            $newItems = [
                [
                    'TEXT' => GetMessage('FERMAOFD_NEW_RECEIPT'),
                    'TITLE' => GetMessage('FERMAOFD_NEW_INCOME_RECEIPT'),
                    'MENU' => [
                        [
                            'TEXT' => GetMessage('FERMAOFD_INCOME_RECEIPT'),
                            'TITLE' => GetMessage('FERMAOFD_NEW_INCOME_RECEIPT'),
                            'ACTION' => 'window.ofdferma.openOrderReceiptPanel("' . $orderId . '");',
                        ], [
                            'TEXT' => GetMessage('FERMAOFD_OUTCOME_RECEIPT'),
                            'TITLE' => GetMessage('FERMAOFD_NEW_OUTCOME_RECEIPT'),
                            'ACTION' => 'window.ofdferma.openOrderRefundPanel("' . $orderId . '");',
                        ],
                    ],
                ],
            ];

            array_splice($items, -2, 0, $newItems);
        }
    }

    public static function onSalePayOrder(Payment $payment, array $oldValues)
    {
        CModule::IncludeModule('sale');

        $enabled = COption::GetOptionString(FERMAOFD_FERMA_MODULE_ID, 'agent_auto_create', 'Y');

        if ($enabled !== 'Y') {
            return null;
        }

        $oldIsPaid = $oldValues['PAID'] === 'Y';
        $newIsPaid = $payment->isPaid();

        if (!$newIsPaid) {
            //     ,   ,    ,      
            return null;
        }

        $order = $payment->getOrder();

        if (!$order) {
            CMainOfdFerma::log(
                GetMessage('FERMAOFD_SENDING_RECEIPT'), GetMessage('FERMAOFD_ORDER') . ' ' . $order->getId(),
                GetMessage('FERMAOFD_FAILED_SENDING_ORDER_NOT_FOUND'), 'WARNING'
            );

            return null;
        }

        try {
            $fermaApiClient = CMainOfdFerma::getApiClient();
        } catch (\RuntimeException $e) {
            CMainOfdFerma::log(
                GetMessage('FERMAOFD_SENDING_RECEIPT'), GetMessage('FERMAOFD_ORDER') . ' ' . $order->getId(),
                GetMessage('FERMAOFD_FAILED_SENDING_FERMA_SETTINGS'), 'WARNING'
            );

            return null;
        }

        $receiptData = $fermaApiClient->prepareElectronReceipt('Income', $order);

        if (empty($receiptData)) {
            CMainOfdFerma::log(
                GetMessage('FERMAOFD_FAILED_RECEIPT_MAKING'), GetMessage('FERMAOFD_ORDER') . ' ' . $order->getId(),
                GetMessage('FERMAOFD_FAILED_RECEIPT_MAKING_UNEXPECTED'), 'ERROR'
            );

            return null;
        }

        CAgent::AddAgent(
            sprintf('COfdFermaAgent::sendReceiptAgent(%s, "%s", %d);', Json::encode($order->getId()), addslashes(Json::encode($receiptData)), time()),
            FERMAOFD_FERMA_MODULE_ID,
            'Y', 30
        );

        return true;
    }

    public static function OnOrderUpdate($id, $fields)
    {
        //   ,         
        if ($fields['DEDUCTED'] === 'Y') {
            CMainOfdFerma::emitFinishReceipts($id);
        }
    }

    public static function onSalePayCancelOrder(Payment $payment, array $oldValues)
    {
        CModule::IncludeModule('sale');

        $enabled = COption::GetOptionString(FERMAOFD_FERMA_MODULE_ID, 'agent_auto_create', 'Y');

        if ($enabled !== 'Y') {
            return null;
        }

        $oldIsPaid = $oldValues['PAID'] === 'Y';
        $newIsPaid = $payment->isPaid();

        if ($newIsPaid) {
            //      ,   ,   ,      
            return null;
        }

        $order = $payment->getOrder();

        if (!$order) {
            CMainOfdFerma::log(
                GetMessage('FERMAOFD_SENDING_RECEIPT'), GetMessage('FERMAOFD_ORDER') . ' ' . $order->getId(),
                GetMessage('FERMAOFD_FAILED_SENDING_ORDER_NOT_FOUND'), 'WARNING'
            );

            return null;
        }

        try {
            $fermaApiClient = CMainOfdFerma::getApiClient();
        } catch (\RuntimeException $e) {
            CMainOfdFerma::log(
                GetMessage('FERMAOFD_SENDING_RECEIPT'), GetMessage('FERMAOFD_ORDER') . ' ' . $order->getId(),
                GetMessage('FERMAOFD_FAILED_SENDING_FERMA_SETTINGS'), 'WARNING'
            );

            return null;
        }

        $receiptData = $fermaApiClient->prepareElectronReceipt('IncomeReturn', $order);

        if (empty($receiptData)) {
            CMainOfdFerma::log(
                GetMessage('FERMAOFD_FAILED_RECEIPT_MAKING'), GetMessage('FERMAOFD_ORDER') . ' ' . $order->getId(),
                GetMessage('FERMAOFD_FAILED_RECEIPT_MAKING_UNEXPECTED'), 'ERROR'
            );

            return null;
        }

        CAgent::AddAgent(
            sprintf('COfdFermaAgent::sendReceiptAgent(%s, "%s", %d);', Json::encode($order->getId()), addslashes(Json::encode($receiptData)), time()),
            FERMAOFD_FERMA_MODULE_ID,
            'Y', 30
        );

        return true;
    }

    public static function emitFinishReceipts($orderId)
    {
        $receiptsQuery = COfdFermaReceipt::GetList(['ID' => 'DESC'], [
            'ORDER_ID' => $orderId,
            'TYPE' => 'Income',
            'PAYMENT_METHOD_ID' => [1, 2, 3],
            'FINISHED' => null,
        ]);

        while ($receipt = $receiptsQuery->Fetch()) {
            $receiptData = Json::decode($receipt['CONTENT']);

            if (empty($receiptData)) {
                CMainOfdFerma::log(
                    GetMessage('FERMAOFD_FAILED_RECEIPT_MAKING'), GetMessage('FERMAOFD_ORDER') . ' ' . $orderId,
                    GetMessage('FERMAOFD_FAILED_RECEIPT_MAKING_UNEXPECTED'), 'ERROR'
                );

                continue;
            }

            $receiptData['InvoiceId'] = CFermaApiService::createInvoiceId($orderId, 'Income');

            foreach ($receiptData['CustomerReceipt']['Items'] as $k => $item) {
                $receiptData['CustomerReceipt']['Items'][$k]['PaymentMethod'] = '4';
            }

            CAgent::AddAgent(
                sprintf('COfdFermaAgent::sendReceiptAgent(%s, "%s", %d);', Json::encode($orderId), addslashes(Json::encode($receiptData)), time()),
                FERMAOFD_FERMA_MODULE_ID,
                'Y', 30
            );

            COfdFermaReceipt::Update($receipt['ID'], ['FINISHED' => 1]);
        }
    }

    /**
     * @deprecated use COfdFermaAgent
     */
    public static function sendReceiptAgent($orderId, $receiptData, $startedAt = null)
    {
        return COfdFermaAgent::sendReceiptAgent($orderId, $receiptData, $startedAt);
    }

    /**
     * @deprecated use COfdFermaAgent
     */
    public static function checkReceiptStatusAgent($receiptId, $startedAt = null)
    {
        return COfdFermaAgent::checkReceiptStatusAgent($receiptId, $startedAt);
    }

    /**
     * @deprecated use COfdFermaAgent
     */
    public static function sendReceiptData($orderId, $receiptData)
    {
        return COfdFermaAgent::sendReceiptData($orderId, $receiptData);
    }

    /**
     * @var \CFermaApiService
     */
    private static $client;

    public static function getApiClient()
    {
        if (null === CMainOfdFerma::$client) {
            $fermaApiHost = COption::GetOptionString(FERMAOFD_FERMA_MODULE_ID, 'api_host', 'https://ferma.ofd.ru');
            $fermaApiLogin = COption::GetOptionString(FERMAOFD_FERMA_MODULE_ID, 'api_login');
            $fermaApiPassword = COption::GetOptionString(FERMAOFD_FERMA_MODULE_ID, 'api_password');
            $agentInn = COption::GetOptionString(FERMAOFD_FERMA_MODULE_ID, 'agent_inn');
            $agentTaxation = COption::GetOptionString(FERMAOFD_FERMA_MODULE_ID, 'agent_taxation_system');
            $errorsEmail = COption::GetOptionString(FERMAOFD_FERMA_MODULE_ID, 'errors_email');

            if (empty($fermaApiHost)
                || empty($fermaApiLogin)
                || empty($fermaApiPassword)
                || empty($agentInn)
                || empty($agentTaxation)
            ) {
                throw new \RuntimeException(GetMessage('FERMAOFD_EMPTY_SETTINGS'));
            }

            CMainOfdFerma::$client = CFermaApiService::instance(
                $fermaApiHost, $fermaApiLogin, $fermaApiPassword,
                $agentInn, $agentTaxation, $errorsEmail
            );
        }

        return CMainOfdFerma::$client;
    }

    public static function log($operation, $entity, $message, $level)
    {
        if (class_exists('CEventLog')) {
            CEventLog::Add([
                'SEVERITY' => $level,
                'AUDIT_TYPE_ID' => $operation,
                'MODULE_ID' => FERMAOFD_FERMA_MODULE_ID,
                'ITEM_ID' => $entity,
                'DESCRIPTION' => $message,
            ]);
        }
    }
}
