<?php

require_once __DIR__ . '/../../const.php';

use Bitrix\Main\Web\Json;
use Bitrix\Sale\Order;

CModule::IncludeModule(FERMAOFD_FERMA_MODULE_ID);

class COfdFermaAgent
{
    public static function sendReceiptAgent($orderId, $receiptData, $startedAt = null)
    {
        CModule::IncludeModule('sale');

        $receiptData = Json::decode($receiptData);

        try {
            $fermaApiClient = CMainOfdFerma::getApiClient();
        } catch (\RuntimeException $e) {
            CMainOfdFerma::log(
                GetMessage('FERMAOFD_SENDING_RECEIPT'), GetMessage('FERMAOFD_ORDER') . ' ' . $orderId,
                GetMessage('FERMAOFD_FAILED_SENDING_FERMA_SETTINGS'), 'WARNING'
            );

            return null;
        }

        try {
            COfdFermaAgent::sendReceiptData($orderId, $receiptData);
        } catch (\RuntimeException $e) {
            CMainOfdFerma::log(
                GetMessage('FERMAOFD_SENDING_RECEIPT'), GetMessage('FERMAOFD_ORDER') . ' ' . $orderId,
                $e->getMessage(), 'WARNING'
            );

            //     
            if ($e->getCode() === 1019) {
                return false;
            }

            if (empty($receiptData)) {
                CMainOfdFerma::log(
                    GetMessage('FERMAOFD_RECEIPT_MAKING_FAILED'), GetMessage('FERMAOFD_ORDER') . ' ' . $orderId,
                    GetMessage('FERMAOFD_FAILED_RECEIPT_MAKING_UNEXPECTED'), 'ERROR'
                );

                return false;
            }

            $order = Order::load($orderId);

            if (!$order) {
                CMainOfdFerma::log(
                    GetMessage('FERMAOFD_SENDING_RECEIPT'), GetMessage('FERMAOFD_ORDER') . ' ' . $orderId,
                    GetMessage('FERMAOFD_FAILED_SENDING_ORDER_NOT_FOUND'), 'WARNING'
                );

                return false;
            }

            if ($startedAt !== null) {
                $passedTime = time() - $startedAt;

                //    24 
                if ($passedTime > 86400) {
                    CMainOfdFerma::log(
                        GetMessage('FERMAOFD_ATTEMPTS_SENDING_ARE_TERMINATED'), GetMessage('FERMAOFD_ORDER') . ' ' . $orderId,
                        GetMessage('FERMAOFD_FAILED_SENDING_PERIODIC'), 'WARNING'
                    );

                    return false;
                }
            }

            $newReceiptData = $fermaApiClient->prepareElectronReceipt($receiptData['Type'], $order);

            return sprintf('CMainOfdFerma::sendReceiptAgent(%s, "%s", %d);', Json::encode($orderId), addslashes(Json::encode($newReceiptData)), $startedAt === null ? time() : $startedAt);
        }

        CMainOfdFerma::log(
            GetMessage('FERMAOFD_SENDING_RECEIPT'), GetMessage('FERMAOFD_ORDER') . ' ' . $orderId,
            GetMessage('FERMAOFD_SUCCESS_SENDING'), 'INFO'
        );

        return false;
    }

    public static function checkReceiptStatusAgent($receiptId, $startedAt = null)
    {
        $receipt = COfdFermaReceipt::GetByID($receiptId);

        if (empty($receipt)) {
            CMainOfdFerma::log(
                GetMessage('FERMAOFD_RECEIPT_CHECKING'), GetMessage('FERMAOFD_RECEIPT') . ' ' . $receiptId,
                GetMessage('FERMAOFD_FAILED_CHECKING_RECEIPT_NOT_FOUND'), 'WARNING'
            );

            return false;
        }

        try {
            $fermaApiClient = CMainOfdFerma::getApiClient();
        } catch (\RuntimeException $e) {
            CMainOfdFerma::log(
                GetMessage('FERMAOFD_RECEIPT_CHECKING'), GetMessage('FERMAOFD_RECEIPT') . ' ' . $receiptId,
                GetMessage('FERMAOFD_FAILED_CHECKING_FERMA_SETTINGS'), 'WARNING'
            );

            return false;
        }

        try {
            $response = $fermaApiClient->getElectronReceiptStatus($receipt['RECEIPT_ID']);
        } catch (CFermaApiException $e) {
            if ($startedAt !== null) {
                $passedTime = time() - $startedAt;

                //    7 
                if ($passedTime > (86400 * 7)) {
                    CMainOfdFerma::log(
                        GetMessage('FERMAOFD_ATTEMPTS_CHECKING_ARE_TERMINATED'), GetMessage('FERMAOFD_RECEIPT') . ' ' . $receiptId,
                        GetMessage('FERMAOFD_FAILED_CHECKING'), 'WARNING'
                    );

                    return false;
                }
            }

            return sprintf('CMainOfdFerma::checkReceiptStatusAgent(%s, %d);', Json::encode($receipt['ID']), $startedAt === null ? time() : $startedAt);
        }

        if ($response['Status'] === 'Failed') {
            if (isset($response['Error']['Message'])) {
                CMainOfdFerma::log(
                    GetMessage('FERMAOFD_RECEIPT_CHECKING'), GetMessage('FERMAOFD_RECEIPT') . ' ' . $receiptId,
                    sprintf(GetMessage('FERMAOFD_FAILED_CHECKING') . ': %s.', $response['Error']['Message']), 'WARNING'
                );
            } else {
                CMainOfdFerma::log(
                    GetMessage('FERMAOFD_RECEIPT_CHECKING'), GetMessage('FERMAOFD_RECEIPT') . ' ' . $receiptId,
                    sprintf(GetMessage('FERMAOFD_FAILED_CHECKING_UNEXPECTED')), 'WARNING'
                );
            }

            return false;
        }

        $data = $response['Data'];

        COfdFermaReceipt::Update($receipt['ID'], [
            'STATUS_ID' => $data['StatusCode'],
            'STATUS_NAME' => $data['StatusName'],
            'STATUS_MESSAGE' => $data['StatusMessage'],
            'DEVICE_ID' => $data['Device']['DeviceId'],
            'RNM' => $data['Device']['RNM'],
            'ZN' => $data['Device']['ZN'],
            'FN' => $data['Device']['FN'],
            'FDN' => $data['Device']['FDN'],
            'FPD' => $data['Device']['FPD'],
        ]);

        CMainOfdFerma::log(
            GetMessage('FERMAOFD_RECEIPT_CHECKING'), GetMessage('FERMAOFD_RECEIPT') . ' ' . $receiptId,
            GetMessage('FERMAOFD_SUCCESS_CHECKING'), 'INFO'
        );

        if ($data['StatusCode'] < 2) {
            if ($startedAt !== null) {
                $passedTime = time() - $startedAt;

                //    7 
                if ($passedTime > (86400 * 7)) {
                    CMainOfdFerma::log(
                        GetMessage('FERMAOFD_ATTEMPTS_CHECKING_ARE_TERMINATED'), GetMessage('FERMAOFD_RECEIPT') . ' ' . $receiptId,
                        GetMessage('FERMAOFD_FAILED_CHECKING'), 'WARNING'
                    );

                    return false;
                }
            }


            return 'CMainOfdFerma::checkReceiptStatusAgent(' . Json::encode($receipt['ID']) . ');';
        }

        //  false     ,    
        return false;
    }

    public static function sendReceiptData($orderId, $receiptData)
    {
        global $DB;

        try {
            $fermaApiClient = CMainOfdFerma::getApiClient();
        } catch (\RuntimeException $e) {
            throw new \RuntimeException(GetMessage('FERMAOFD_FAILED_SENDING_FERMA_SETTINGS'));
        }

        try {
            $response = $fermaApiClient->sendElectronReceipt($receiptData);
        } catch (CFermaApiException $e) {
            throw new \RuntimeException(sprintf(GetMessage('FERMAOFD_FAILED_SENDING') . ': %s.', $e->getMessage()));
        }

        if (empty($response)) {
            throw new \RuntimeException(GetMessage('FERMAOFD_FAILED_SENDING_FERMA_EMPTY_RESPONSE'));
        }

        if ($response['Status'] === 'Failed') {
            if (!isset($response['Error']['Message'])) {
                throw new \RuntimeException(GetMessage('FERMAOFD_FAILED_SENDING_UNEXPECTED'));
            }

            throw new \RuntimeException(sprintf(GetMessage('FERMAOFD_FAILED_SENDING') . ': %s.', $response['Error']['Message']), $response['Error']['Code']);
        }

        $total = 0;

        foreach ($receiptData['CustomerReceipt']['Items'] as $item) {
            $total += $item['Amount'];
        }

        $id = COfdFermaReceipt::Add([
            'SITE_ID' => SITE_ID,
            'RECEIPT_ID' => $response['Data']['ReceiptId'],
            'ORDER_ID' => $orderId,
            'PAYMENT_METHOD_ID' => $receiptData['CustomerReceipt']['Items'][0]['PaymentMethod'],
            'CONTENT' => Json::encode($receiptData),
            'TYPE' => $receiptData['Type'],
            'TOTAL_PRICE' => $total,
            'ENV' => $fermaApiClient->getEnvId(),
            '=DATE_CREATE' => $DB->GetNowFunction(),
            '=DATE_UPDATE' => $DB->GetNowFunction(),
        ]);

        CAgent::AddAgent(
            'CMainOfdFerma::checkReceiptStatusAgent(' . Json::encode($id) . ');',
            FERMAOFD_FERMA_MODULE_ID,
            'Y', 10
        );

        return true;
    }
}
